--[[
	Various additions to JugadorXEI's LUA_MUSH lua script, with simple air
	control and sanity checks.
]]

local grill = 4

local MR_CENTER = 1 -- 2nd highest bounce
local MR_SLOPE	= 2 -- highest bounce
local MR_DIP	= 3 -- lowest bounce
local MR_PERIMETER = 4 -- 3rd highest bounce
local MR_EDGE	= 5 -- tas lowest bounce

freeslot("sfx_mpbing")

local FRACUNIT		= FRACUNIT
local ITEMTIME		= 8 * TICRATE
local ACCELMAX 		= 4000
local ORIG_FRICTION = 62914

local BT_ACCELERATE = BT_ACCELERATE
local BT_BRAKE 		= BT_BRAKE
local BT_DRIFT 		= BT_DRIFT

local ANG1		= ANG1
local ANGLE_45 	= ANGLE_45
local ANGLE_135 = ANGLE_135

local EASYSPEED 	= 0
local NORMALSPEED 	= 1
local HARDSPEED 	= 2

local MAXSKINCOLORS = MAXSKINCOLORS

local function CheckPlayerWasLastSextor(p)
	if not (p and p.mo) then return end
	p.mushrepeats = $ or 0
	p.mushsec = $ or nil
	if p.mushrepeats > grill - 1 then
		S_StartSound(p.mo, sfx_mpbing)
		P_KillMobj(p.mo) -- kill yourself, NOW
		p.mushsec = nil
		p.mushrepeats = 0
	end
	if p.deadtimer > 0 then
		p.mushsec = nil
		p.mushrepeats = 0
	end
end

-- another edited version of a function that really does not need an onground check :)))))))
local function K_GetKartSpeed(player, doboostpower)
	local k_speed = 150
	local g_cc = FRACUNIT
	local xspd = 3072
	local kartspeed = player.kartspeed
	local finalspeed

	if gamespeed == EASYSPEED then
		g_cc = 53248 + xspd -- 50cc =  81.25 + 4.69 =  85.94%
	elseif gamespeed == HARDSPEED then
		g_cc = 77824 + xspd -- 150cc = 118.75 + 4.69 = 123.44%
	else -- NORMALSPEED
		g_cc = 65536 + xspd -- 100cc = 100.00 + 4.69 = 104.69%
	end

	if G_BattleGametype() and player.kartstuff[k_bumper] <= 0 then
		kartspeed = 1
	end

	k_speed = $ + kartspeed * 3 -- 153 - 177

	finalspeed = FixedMul(FixedMul(k_speed << 14, g_cc), player.mo.scale);

	if doboostpower then
		return FixedMul(finalspeed, player.kartstuff[k_boostpower] + player.kartstuff[k_speedboost])
	end

	return finalspeed
end

local function accelcode(player, playerMo)
	if not player.mushturning then return end
	local playerButtons = player.cmd.buttons

	-- Turncode	
	local playerAngle = playerMo.angle
	local thrustAngle = playerAngle

	-- Only apply turnrate if the player has a minimum of speed.
	if player.speed > (20 * FRACUNIT) then
		-- naturally correct this momentum angle to the player's
		local thrustAngle = R_PointToAngle2(0, 0, playerMo.momx, playerMo.momy)
	
		-- turn rate calC
		local angleCorrectRate = FixedDiv((ANGLE_45 + ANG1) - (ANG1 * player.kartweight), FRACUNIT * 35)
	
	
		local angleMovementResult = angleCorrectRate
		local angleSidewaysMovementDiff = thrustAngle - playerAngle
		local angleAbsSidewaysMovement = abs(angleSidewaysMovementDiff)
	
		if angleAbsSidewaysMovement < angleCorrectRate then
			angleMovementResult = angleAbsSidewaysMovement
			-- print("angle less than one, correcting.")
		end
	
		if angleSidewaysMovementDiff > 0 then
			thrustAngle = $ - angleMovementResult
		elseif angleSidewaysMovementDiff < 0 then
			thrustAngle = $ + angleMovementResult
		end
	
		local thrustForce = R_PointToDist2(0, 0, playerMo.momx, playerMo.momy)

		-- let the player control themselves horizontally
		playerMo.momx = FixedMul(thrustForce, cos(thrustAngle))
		playerMo.momy = FixedMul(thrustForce, sin(thrustAngle))
	end

	-- Accelcode.
	local speedValue = K_GetKartSpeed(player, true)
	local accelValue = K_GetKartAccel(player)

	local oldSpeed = R_PointToDist2(0, 0, player.rmomx, player.rmomy) -- FixedMul(P_AproxDistance(player.rmomx, player.rmomy), player.mo.scale);
	local newSpeed = FixedDiv(FixedDiv(FixedMul(oldSpeed, ACCELMAX - accelValue) +
		FixedMul(speedValue, accelValue), ACCELMAX), ORIG_FRICTION)

	local finalSpeed = newSpeed - oldSpeed

	-- forwardmove is:
	--  50 while accelerating,
	--  25 while clutching,
	--   0 with no gas, and
	-- -25 when only braking.

	local forwardmove 	= 0
	local holdingGas 	= (playerButtons & BT_ACCELERATE) == BT_ACCELERATE
	local holdingBrake 	= (playerButtons & BT_BRAKE) == BT_BRAKE
	if holdingGas and not holdingBrake then -- Accelerating
		forwardmove = 50
	elseif holdingGas and holdingBrake then -- Clutching
		forwardmove = 25
	elseif not holdingGas and holdingBrake then -- Braking
		forwardmove = -25
	end

	finalSpeed = $ * (forwardmove/25)
	finalSpeed = $ / 2

	if forwardmove < 0 and finalSpeed > FRACUNIT*2 then
		finalSpeed = finalSpeed/2
	elseif forwardmove < 0 then
		finalSpeed = -FRACUNIT/2
	end

	if finalSpeed < 0 then finalSpeed = 0 end

	playerMo.momx = $ + P_ReturnThrustX(thrustAngle, finalSpeed)
	playerMo.momy = $ + P_ReturnThrustY(thrustAngle, finalSpeed)

	playerMo.momx = FixedMul(playerMo.momx, playerMo.friction)
	playerMo.momy = FixedMul(playerMo.momy, playerMo.friction)
end

local function RealThink()
	if leveltime <= 2 then
		for p in players.iterate do
			p.mushrepeats = 0
			p.mushsec = nil
			p.mushturning = false
		end
	end
	for p in players.iterate do
		if not (p.mo and p.mo.valid) then continue end
		if P_IsObjectOnGround(p.mo) then
			p.mushturning = false
		end
		CheckPlayerWasLastSextor(p)
		accelcode(p, p.mo)
		if p.mushturning then
			local spindiff = ( ANGLE_22h * leveltime ) --TODO
			p.frameangle = $ + spindiff
		end
	end
end
addHook("PostThinkFrame", RealThink)

addHook("LinedefExecute",
function(line, playerMo, sector)
	if playerMo.player.mushsec and playerMo.player.mushsec == playerMo.player.mo.subsector.sector then
		playerMo.player.mushrepeats = $ + 1
	end
	jug_mkwiimushrooms.wiiMushrooms(line, playerMo, sector, MR_CENTER)
	playerMo.player.mushsec = playerMo.player.mo.subsector.sector
	playerMo.player.mushturning = true
	
end, "MUSHCUSTCENTER")
	
addHook("LinedefExecute",
function(line, playerMo, sector)
	if playerMo.player.mushsec and playerMo.player.mushsec == playerMo.player.mo.subsector.sector then
		playerMo.player.mushrepeats = $ + 1
	end
	jug_mkwiimushrooms.wiiMushrooms(line, playerMo, sector, MR_SLOPE)
	playerMo.player.mushsec = playerMo.player.mo.subsector.sector
	playerMo.player.mushturning = true
end, "MUSHCUSTSLOPE")
	
addHook("LinedefExecute",
function(line, playerMo, sector)
	if playerMo.player.mushsec and playerMo.player.mushsec == playerMo.player.mo.subsector.sector then
		playerMo.player.mushrepeats = $ + 1
	end
	jug_mkwiimushrooms.wiiMushrooms(line, playerMo, sector, MR_DIP)
	playerMo.player.mushsec = playerMo.player.mo.subsector.sector
	playerMo.player.mushturning = true
end, "MUSHCUSTDIP")
	
addHook("LinedefExecute",
function(line, playerMo, sector)
	if playerMo.player.mushsec and playerMo.player.mushsec == playerMo.player.mo.subsector.sector then
		playerMo.player.mushrepeats = $ + 1
	end
	jug_mkwiimushrooms.wiiMushrooms(line, playerMo, sector, MR_PERIMETER)
	playerMo.player.mushsec = playerMo.player.mo.subsector.sector
	playerMo.player.mushturning = true
end, "MUSHCUSTPERIMETE")
	
addHook("LinedefExecute",
function(line, playerMo, sector)
	if playerMo.player.mushsec and playerMo.player.mushsec == playerMo.player.mo.subsector.sector then
		playerMo.player.mushrepeats = $ + 1
	end
	jug_mkwiimushrooms.wiiMushrooms(line, playerMo, sector, MR_EDGE)
	playerMo.player.mushsec = playerMo.player.mo.subsector.sector
	playerMo.player.mushturning = true
end, "MUSHCUSTEDGE")

