local fadesectors = {}

addHook("ThinkFrame", function()
    -- Going backward so we can remove entries while iterating through list
    for i = #fadesectors, 1, -1 do
        local sec_data = fadesectors[i]

        local sector = sec_data.sector
        local fade_target = sec_data.fade_target
        local step = sec_data.fade_step
        local bound = step > 0 and min or max

        if not (sector and sector.valid) then
            table.remove(fadesectors, i)
        else
            sec_data.light = sec_data.light + step
            sector.lightlevel = bound(sec_data.light/FRACUNIT, fade_target)

            if sector.lightlevel == fade_target then
                table.remove(fadesectors, i)
            end
        end
    end
end)

local function addSectorFade(sector, fade_target, fade_step)
    for i = 1, #fadesectors do
        local sec_data = fadesectors[i]

        if #sec_data.sector == #sector then
            sec_data.fade_target = fade_target
            sec_data.fade_step = fade_step
            return
        end
    end

    table.insert(fadesectors, {
        sector = sector,
        light = sector.lightlevel*FRACUNIT,
        fade_target = fade_target,
        fade_step = fade_step,
    })
end

rawset(_G, "LUA_addSectorFade", addSectorFade)

addHook("LinedefExecute", function(line, mobj, sector)
    local fade_target = line.frontside.textureoffset/FRACUNIT -- X offset
    local fade_step = line.frontside.rowoffset -- Y offset
    local usetag = line.flags & ML_EFFECT2 -- No midtexture scew
    local perplayer = line.flags & ML_EFFECT3 -- Peg midtexture

    if line.backside and line.backside.rowoffset then
        fade_step = FixedDiv(fade_step, line.backside.rowoffset) -- Backside Y offset
    end

    for sector in sectors.iterate do
        if (not usetag or line.tag == sector.tag) and (not perplayer or (mobj and mobj.player and mobj.player == consoleplayer)) then
            addSectorFade(sector, fade_target, fade_step)
        end
    end
end, "SECFADE")
